searchNodes=[{"ref":"PgMoney.html","title":"PgMoney","type":"module","doc":"Contains the all the basic types and guards to work with the money data type."},{"ref":"PgMoney.html#is_money/1","title":"PgMoney.is_money/1","type":"macro","doc":"Returns true if value is an integer and falls between the minimum/0 and maximum/0 (inclusive) range of the money data type."},{"ref":"PgMoney.html#is_precision/1","title":"PgMoney.is_precision/1","type":"macro","doc":"Returns true if value is a valid precision/0, false otherwise."},{"ref":"PgMoney.html#is_telemetry/1","title":"PgMoney.is_telemetry/1","type":"macro","doc":"Returns true if value is a valid telemetry/0, false otherwise."},{"ref":"PgMoney.html#max/1","title":"PgMoney.max/1","type":"function","doc":"Returns the maximum Decimal.t() value for the given precision."},{"ref":"PgMoney.html#maximum/0","title":"PgMoney.maximum/0","type":"function","doc":"The maximum integer value possible for the money data type."},{"ref":"PgMoney.html#min/1","title":"PgMoney.min/1","type":"function","doc":"Returns the minimum Decimal.t() value for the given precision."},{"ref":"PgMoney.html#minimum/0","title":"PgMoney.minimum/0","type":"function","doc":"The minimum integer value possible for the money data type."},{"ref":"PgMoney.html#storage_size/0","title":"PgMoney.storage_size/0","type":"function","doc":"The storage size the money data type takes up in the database."},{"ref":"PgMoney.html#t:config/0","title":"PgMoney.config/0","type":"type","doc":""},{"ref":"PgMoney.html#t:money/0","title":"PgMoney.money/0","type":"type","doc":""},{"ref":"PgMoney.html#t:precision/0","title":"PgMoney.precision/0","type":"type","doc":""},{"ref":"PgMoney.html#t:telemetry/0","title":"PgMoney.telemetry/0","type":"type","doc":""},{"ref":"PgMoney.Extension.html","title":"PgMoney.Extension","type":"module","doc":"Implements how to encode and decode PostgeSQL&#39;s money data type."},{"ref":"PgMoney.Extension.html#to_dec/3","title":"PgMoney.Extension.to_dec/3","type":"function","doc":"Returns a Decimal.t/0 which corresponds to money with given precision."},{"ref":"PgMoney.Extension.html#to_int/3","title":"PgMoney.Extension.to_int/3","type":"function","doc":"Returns an integer which corresponds to money with given precision."},{"ref":"readme.html","title":"README","type":"extras","doc":"README"},{"ref":"readme.html#intro-short","title":"README - Intro Short","type":"extras","doc":"Adds PostgreSQL&#39;s money data type support to Postgrex."},{"ref":"readme.html#intro-long","title":"README - Intro Long","type":"extras","doc":"I do integrations, to be more specific, I help my customer to migrate his current ERP-system. The target database is a PostgreSQL database and involves many tables using the money data type. To my surprise Postgrex doesn&#39;t support this data type... for a good reason. Hence the need for this library."},{"ref":"readme.html#but","title":"README - But!","type":"extras","doc":"The usage of money should be avoided though as explained in the PostgreSQL wiki&#39;s Don&#39;t do this: Don&#39;t use money The money data type isn&#39;t actually very good for storing monetary values. Numeric, or (rarely) integer may be better. Why not? lots of reasons. It&#39;s a fixed-point type, implemented as a machine int, so arithmetic with it is fast. But it doesn&#39;t handle fractions of a cent (or equivalents in other currencies), it&#39;s rounding behaviour is probably not what you want. It doesn&#39;t store a currency with the value, rather assuming that all money columns contain the currency specified by the database&#39;s lc_monetary locale setting. If you change the lc_monetary setting for any reason, all money columns will contain the wrong value. That means that if you insert $10.00 while lc_monetary is set to en_US.UTF-8 the value you retrieve may be 10,00 Lei or ¥1,000 if lc_monetary is changed. Storing a value as a numeric, possibly with the currency being used in an adjacent column, might be better. When should you? If you [ ] are only working in a single currency [ ] aren&#39;t dealing with fractional cents [ ] are only doing addition and subtraction then money might be the right thing. ... so probably never. Read Mathias Verraes&#39; excellent chapter Emergent Contexts through Refinement in DDD - The First 15 Years to get a better understanding of currencies. But this flawed type is out there and needs some interfacing, hence this project. See how it fails you in a fail in three acts."},{"ref":"readme.html#installation","title":"README - Installation","type":"extras","doc":"Option 1, hex.pm If available in Hex, the package can be installed by adding pg_money to your list of dependencies in mix.exs: def deps do [ {:pg_money, &quot;~&gt;0.4.20&quot;} ] end Option 2, github Reference it via it&#39;s github url: def deps do [ {:pg_money, git: &quot;https://github.com/CodeJordan23/pg_money.git&quot;, tag: &quot;0.4.20&quot;} ] end"},{"ref":"readme.html#usage","title":"README - Usage","type":"extras","doc":"Postgrex only my_types = [ {PgMoney.Extension, [precision: 2, telemetry_prefix: [:my, :prefix]]} ] Postgrex.Types.define(MyApp.PostgresTypes, my_types, []) opts = [hostname: &quot;localhost&quot;, username: &quot;postgres&quot;, database: &quot;pg_money_test&quot;, types: MyApp.PostgresTypes ] # or use PgMoney.Type {:ok, pid} = Postgrex.Connection.start_link(opts) with Ecto You will want to add a new file with your type definition like postgres_types.ex Postgrex.Types.define(MyApp.PostgresTypes, [{PgMoney.Extension, [precision: 2, telemetry_prefix: [:my, :prefix]]}] ++ Ecto.Adapters.Postgres.extensions(), []) and then configure your repository to use it like so: config :my_app, MyApp.Repo, types: MyApp.PostgresTypes Head over to Ecto.Adapaters.Postgres&#39; module extension to learn more."},{"ref":"readme.html#license","title":"README - License","type":"extras","doc":"Copyright 2019 Michael J. Lüttjohann Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License."},{"ref":"readme.html#and-last-but-not-least","title":"README - And last but not least","type":"extras","doc":"If I have seen further it is by standing on the shoulders of Giants. - Isaac Newton I want to mention some of these giants and warmly thank all the people involved: PostgreSQL a great and free database. The well written documentation helped me to write the kick-starter tests for this library. Postgrex For making it so easy to extend. Credo and mix format for giving me guidance. Plataformatec for hooking me on Elixir with their little Ecto Cookbook on August 19th. 13 weeks ago... Mathias Verraes found at his homepage for writing his superb article Emergent Contexts through Refinement found here DDD - The First 15 Years and like all the people involved with this book. the whole Elixir ecosystem. It was fun writing this library. writing documentation was easy and generated easily with ExDoc writing tests was seamless. publishing to hexdocs.pm seems straigtforward github.com and hexdocs.pm for hosting this library."},{"ref":"fail_in_three_acts.html","title":"A Fail In Three Acts","type":"extras","doc":"A Fail In Three Acts As stated in the README.md the money data type is flawed. Basically it is a 64-bit signed integer and the connection&#39;s current lc_monetary settings dictates how to interpret it. Let&#39;s see what this means:"},{"ref":"fail_in_three_acts.html#analysis","title":"A Fail In Three Acts - Analysis","type":"extras","doc":"Preludium BEGIN TRANSACTION; CREATE TEMP TABLE money_test ( m money , n numeric ) ON COMMIT DROP; SHOW lc_monetary; Result: German_Germany.1252 Act I INSERT INTO money_test ( m , n ) VALUES ( 12345.6789 , 12345.6789 ); SELECT * FROM money_test; Result: | m | n | |------------:|-----------:| | 12.345,68 € | 12345.6789 | Interludium SET lc_monetary to &#39;Arabic_Bahrain&#39;; -- because I am coding on a windows machine -- use &#39;ar_BH.utf8&#39; on anything else, but no warranty. -- for this example anything that has another precision than your database&#39;s default will do. -- My cofinguration is two digit preecision, so here I&#39;ll switch over to three. SHOW lc_monetary; Result: Arabic_Bahrain Act II INSERT INTO money_test ( m , n ) VALUES ( 12345.6789 , 12345.6789 ); SELECT * FROM money_test; Result: | m | n | |-----------:|-----------:| | 1,234.568 | 12345.6789 | | 12,345.679 | 12345.6789 | Act III RESET lc_monetary; -- switch back SELECT * FROM money_test; Result: | m | n | |-------------:|-----------:| | 12.345,68 € | 12345.6789 | | 123.456,79 € | 12345.6789 | Le fin ROLLBACK;"},{"ref":"fail_in_three_acts.html#conclusion","title":"A Fail In Three Acts - Conclusion","type":"extras","doc":"As a developer we should not mess with lc_monetary, change it during the application&#39;s life time and respect the database defaults. Furthermore, when using Postgrex or Ecto, all extensions need to be set up at compile time."},{"ref":"lessons_learned.html","title":"Lessons learned","type":"extras","doc":"Lessons learned"},{"ref":"lessons_learned.html#how-to-create-a-postgrex-extension","title":"Lessons learned - How to create a Postgrex extension","type":"extras","doc":"add {:postgrex, &quot;&gt;= 0.0.0&quot;} to mix.exs implement Postgrex.Extension&#39;s callbacks Postgrex.Extension.init/1 Postgrex.Extension.matching/1 Postgrex.Extension.format/1 Postgrex.Extension.encode/1 Postgrex.Extension.decode/1 enable during compile time Postgrex.Types.define( PgMoney.PostgresTypes, [PgMoney.Extension], [] ) see Postgrex.Types.define/3 for more."},{"ref":"lessons_learned.html#how-to-do-property-based-testing","title":"Lessons learned - How to do property-based testing","type":"extras","doc":"add {:propcheck, &quot;~&gt; 1.1&quot;, only: [:test, :dev]} to your mix dependencies. in your tests, for example property_test.exs: defmodule My.PropertyTest do use PropCheck setup do test_ctx = nil test_ctx end property &quot;name&quot;, prop_cfg, test_ctx do # do prop-testing end end do property-based testing :P understood how PropCheck.collect/3 works: test = expected == actual test |&gt; collect( with_title(&quot;how to collect and set a header&quot;), my_sample ) |&gt; collect( with_title(&quot;chain and collect&quot;), more_samples ) To my first surprise this results in: chain and collect ... how to collect and set a header ... Turns out to be right, because the equivalent looks like this: collect( collect( test, with_title(&quot;how to collect and set a header&quot;), my_sample ), with_title(&quot;chain and collect&quot;), more_samples ) properties I tested: symmetry or there and back again: # lib level expected = 2 ^expected = expected |&gt; PgMoney.Extension.to_dec() |&gt; PgMoney.Extension.to_int() # db level # saving involves (hopefully) encoding and decoding [m, n] = DB.save_in_temp_table(conn, decimal) assert Decimal.eq?(decimal, n) assert Decimal.eq?(decimal, m) test oracle and different paths, same destination [m, n] = DB.save_in_temp_table(conn, decimal) could also be seen as a test oracle or different paths, same destination. This is what happens: [insert] | | [store as money] [store as numeric] | | [read] | | m == n We can build upon Postgrex handling of Decimal as a PostgreSQL numeric and compare it with the corresponding moneyvalue. Based on this principle, or at least how I implemented it, we go different paths and arrive at the same destination. more: Choosing properties for property-based testing"},{"ref":"lessons_learned.html#how-to-do-telemetry","title":"Lessons learned - How to do telemetry","type":"extras","doc":"add {:telemetry, &quot;~&gt; 0.4.0&quot;} to your mix dependencies. emit an event with :telemetry.execute/2 or :telemetry.execute/3 to listen to an event you have to attach a handler via :telemetry.attach/4 or :telemetry.attach_many/4 if there is no interest in receiving the attached events you :telemetry.detach/1 your :telemetry.handler_id() if you want to correctly measure elapsed time use :erlang.monotonic_time/0 and :erlang.monotonic_time/1 based on my test I would like to remove the :start and :stop events as the duration is always 0 ns on my computer. read more on Samuel Mullen&#39;s blog post The &quot;How&quot;s, &quot;What&quot;s, and &quot;Why&quot;s of Elixir Telemetry"},{"ref":"lessons_learned.html#follow-up","title":"Lessons learned - Follow up","type":"extras","doc":"learn when :erlang does a process context switch. in my humble understanding that I have right now, I wanted to avoid having this during instrumentation. Hence I emit the :start and :stop events inside the after block like this: def to_dec(integer, precision, telemetry) do started_at = current_time() try do case {PgMoney.is_money(integer), PgMoney.is_precision(precision)} do # implementation end after duration = time_diff(started_at, current_time()) emit_start(telemetry, :to_dec, started_at) emit_stop(telemetry, :to_dec, duration) end end extend and write better properties Right now I have only touched the surface with within_precision_test.exs. The next steps would be to: write properties relating to lossy operations generalize it to any precision I built the foundation to implement a ledger who tracks rounding losses like laid out in Mathias Verraes&#39; article Emergent Contexts through Refinement found in DDD - The First 15 Years. But this is a topic for another time."}]